#include <unistd.h>
#include <sys/wait.h>
#include <stdexcept>
#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <cstdarg>
#include <cstdlib>
#include <cstring>
#include <cerrno>
using namespace std;

ifstream fi;
ofstream fo;

bool verbose = false;

class spawn {
	int write_pipe[2];
	int read_pipe[2];
	int child_pid;

	char read_buf[1024];
	int read_pos = 0, read_len = 0;
public:
	spawn(char * const argv[]) {
		if (pipe(write_pipe) == -1) throw runtime_error("Failed to create pipe");
		if (pipe(read_pipe) == -1) throw runtime_error("Failed to create pipe");
		child_pid = fork();
		if (child_pid == -1) throw runtime_error("Failed to fork process");
		if (child_pid == 0) { // in child
			if (dup2(write_pipe[0], STDIN_FILENO) == -1) throw runtime_error("Failed to dup handle");
			if (dup2(read_pipe[1], STDOUT_FILENO) == -1) throw runtime_error("Failed to dup handle");
			if (close(write_pipe[0]) == -1) throw runtime_error("Failed to close handle");
			if (close(write_pipe[1]) == -1) throw runtime_error("Failed to close handle");
			if (close(read_pipe[0]) == -1) throw runtime_error("Failed to close handle");
			if (close(read_pipe[1]) == -1) throw runtime_error("Failed to close handle");
			if (execv(argv[0], argv) == -1) {
				cerr << "Error launching testee: " << strerror(errno) << "\n";
				exit(1);
			}
		} else { // in parent
			if (close(write_pipe[0]) == -1) throw runtime_error("Failed to close handle");
			if (close(read_pipe[1]) == -1) throw runtime_error("Failed to close handle");
			signal(SIGPIPE, SIG_IGN); // don't let childs's death kill the parent
		}
	}

	bool send_line(int n, const char *buf) {
		if (verbose) {
			cerr << "-> " << buf;
		}
		return write(write_pipe[1], buf, n) == n;
	}

	bool send_line(const char *fmt, ...) {
		char buf[1000];
		va_list ap;
		va_start(ap, fmt);
		int n = vsnprintf(buf, sizeof(buf), fmt, ap);
		if (n < 0) throw runtime_error("Invalid format in vsnprintf");
		if (n >= sizeof(buf)) throw runtime_error("Buffer overflow in vsnprintf");
		va_end(ap);
		return send_line(n, buf);
	}

	bool send_eof() {
		return close(write_pipe[1]) != -1;
	}

	bool recv_line(char buf[], int size) {
		for (int n = 0; ; ++n) {
			if (n + 1 == size) {
				buf[n] = 0;
				if (verbose) {
					cerr << "<- " << buf << "...\n";
				}
				return true;
			}
			if (read_pos >= read_len) {
				read_pos = 0;
				read_len = read(read_pipe[0], read_buf, sizeof(read_buf));
			}
			if (read_len < 0) {
				return false;
			}
			if (read_len == 0) {
				buf[n] = 0;
				if (verbose) {
					cerr << "<- " << buf << "\n";
				}
				return true;
			}
			buf[n] = read_buf[read_pos++];
			if (buf[n] == '\n') {
				buf[n] = 0;
				if (verbose) {
					cerr << "<- " << buf << "\n";
				}
				return true;
			}
		}
	}

	int wait() {
		int status;
		waitpid(child_pid, &status, 0);
		return status;
	}
};

int done(int x, const char * y) {
	fo << x << '\n';
	fo << y << '\n';
	if (verbose) {
		cerr << "Res: " << x << '\n';
		cerr << "Msg: " << y << '\n';
	}
	return 0;
}

int main(int argc, char *argv[]) {
	if (argc < 4) {
		cerr << "Usage: " << argv[0] << " [-v] input output child [params ...]'n";
		return 0;
	}

	int pos = 1;
	if (argv[pos] == string("-v")) {
		verbose = true;
		++pos;
	}

	fi.open(argv[pos++]);
	fo.open(argv[pos++]);

	int n, k;
	fi >> n >> k;
	vector<int> p(n);
	for (int i = 0; i < n; ++i) {
		fi >> p[i];
	}
	if (verbose) {
		cerr << n << " " << k << ":";
		for (int i = 0; i < n; ++i) {
			cerr << " " << p[i];
		}
		cerr << "\n";
	}
	vector<int> r(n);
	for (int i = 0; i < n; ++i) {
		r[--p[i]] = i;
	}

	spawn prog(argv + pos);
	if (!prog.send_line("%d %d\n", n, k)) {
		cerr << "Error initializing testee: " << strerror(errno) << "\n";
		return done(0, "Presentation error");
	}
	int queries = 0;
	for (int i = 1; ; ++i) {
		char buf[10000];
		if (!prog.recv_line(buf, sizeof(buf))) {
			cerr << "Error reading from testee: " << strerror(errno) << "\n";
			return done(0, "Presentation error");
		}
		istringstream is(buf);
		char q; is >> q;
		if (!is) {
			if (queries > 0) { // OK, answer given and no more input from testee
				break;
			}
			return done(0, "Presentation error");
		}
		if (q == '?') { // query
			if (queries > 0) { // answer already given
				return done(0, "Presentation error");
			}
			if (i > k) {
				return done(0, "Too many queries");
			}
			int l; is >> l;
			if (!is || 1 > l || l > n || l % 2 != 1) {
				return done(0, "Presentation error");
			}
			vector<int> u(n);
			for (int j = 0; j < l; ++j) {
				int a; is >> a;
				if (!is || 1 > a || a > n || u[--a]) {
					return done(0, "Presentation error");
				}
				u[a] = 1;
			}
			if (is >> q) {
				return done(0, "Presentation error");
			}
			vector<int> v(l);
			for (int k = 0, j = 0; j < n; ++j) {
				if (u[r[j]]) {
					v[k++] = j;
				}
			}
			int x = r[v[l / 2]] + 1;
			if (!prog.send_line("%d\n", x)) {
				cerr << "Error responding to testee: " << strerror(errno) << "\n";
				return done(0, "Presentation error");
			}
		} else if (q == '!') { // answer
			if (queries > 0) { // one answer already given
				return done(0, "Presentation error");
			}
			queries = i;
			vector<int> a(n);
			for (int j = 0; j < n; ++j) {
				is >> a[j];
				if (!is || 1 > a[j] || a[j] > n) {
					return done(0, "Presentation error");
				}
			}
			if (is >> q) {
				return done(0, "Presentation error");
			}
			for (int j = 0; j < n; ++j) {
				if (a[j] != p[j] + 1) {
					return done(0, "Wrong answer");
				}
			}
		} else { // invalid
			return done(0, "Presentation error");
		}
	}

	if (!prog.send_eof()) {
		cerr << "Error shutting down testee: " << strerror(errno) << "\n";
		return done(0, "Presentation error");
	}
	if (verbose) {
		cerr << "Queries: " << queries << "\n";
	}
	done(1, "OK");
	return prog.wait(); // pass child's status back to environment
}
